#!/bin/bash
######################################################################
#                                                                    #
#  Copyright (c) 2002, 2013 NoMachine, http://www.nomachine.com.     #
#                                                                    #
#  All rights reserved.                                              #
#                                                                    #
######################################################################

RunDir=${0%nxlaunchctl.sh}
RestrictedDir=$(cd "${RunDir}" && pwd)

PARAMS_COUNT_MIN=3
PARAMS_COUNT_MAX=5

. "${RestrictedDir}"/nxfunct.sh

isRunByServer
isRunByUserNX

if [ -e "/bin/launchctl" ];
then
  COMMAND_LAUNCHCTL="/bin/launchctl"
elif [ -e "/usr/bin/launchctl" ];
then
  COMMAND_LAUNCHCTL="/usr/bin/launchctl"
else
  echo "ERROR: launchctl command not found"
  exit 1
fi

COMMAND=$3
SERVICE=$4
USERNAME=$5

if [ "x$SERVICE" = "xlocalnxserver" ]; 
then
  SERVICE_NAME="localnxserver"
  SERVICE_PATH="/Library/LaunchAgents/com.nomachine.localnxserver.plist"
  SERVICE_SOURCE_PATH="/Applications/NoMachine.app/Contents/Frameworks/share/mac/launchd/com.nomachine.localnxserver.plist"
else
  SERVICE_NAME="nxhtd"
  SERVICE_PATH="/Library/LaunchDaemons/com.nomachine.nxhtd.plist"
fi

isRunning()
{
  RES=`${COMMAND_SU} -l ${USERNAME} -c "$COMMAND_LAUNCHCTL list" | $COMMAND_GREP $SERVICE_NAME | $COMMAND_AWK '{print $1}'`

  if [ "x$RES" = "x-" ];
  then
    SESSION_TYPE="Aqua"

    if [ "x${USERNAME}" = "xroot" ];
    then
      SESSION_TYPE="LoginWindow";
    fi

    ${COMMAND_SU} -l ${USERNAME} -c "${COMMAND_LAUNCHCTL} unload -S ${SESSION_TYPE} ${SERVICE_PATH}"
  fi

  RES=`${COMMAND_SU} -l ${USERNAME} -c "${COMMAND_LAUNCHCTL} list" | $COMMAND_GREP $SERVICE_NAME`

  if [ "x$RES" != "x" ];
  then
    return 1;
  else
    return 0;
  fi
}

isDaemonRunning()
{
  RES=`$COMMAND_LAUNCHCTL list | $COMMAND_GREP $SERVICE_NAME | $COMMAND_AWK '{print $1}'`

  if [ "x$RES" = "x-" ];
  then
    ${COMMAND_LAUNCHCTL} unload ${SERVICE_PATH}
  fi

  RES=`${COMMAND_LAUNCHCTL} list | $COMMAND_GREP $SERVICE_NAME`

  if [ "x$RES" != "x" ];
  then
    return 1;
  else
    return 0;
  fi
}

copyFile()
{
  if test ! -f "${SERVICE_PATH}";
  then
    command="${COMMAND_CP} -f '${SERVICE_SOURCE_PATH}' '${SERVICE_PATH}'"
    result=`/bin/bash -c "${command}" 2>&1`
  else
    ${COMMAND_ECHO} "NX> 900 File:'${SERVICE_PATH}' already exists."
    return 0
  fi

  if test $? != 0 -o "x${result}" != "x";
  then
    errorMsg "Problems while coping: '${SERVICE_SOURCE_PATH}' to '${SERVICE_PATH}.backup': [$!][${result}]." "1"
  else
    ${COMMAND_ECHO} "NX> 900 File:'${SERVICE_SOURCE_PATH}' copied in '${SERVICE_PATH}' with no error result:[$!][${result}] exiting with:[$?]."
  fi

  return 0
}

loadFile()
{
  if [ ! -f "${SERVICE_PATH}" ];
  then
    errorMsg "Local session plist file do not exist." "1"
  fi

  isRunning

  RUNNING=$?

  if [ "$RUNNING" -ne 0 ];
  then
    errorMsg "Local session is already loaded." "1"
  fi

  SESSION_TYPE="Aqua"

  if [ "x${USERNAME}" = "xroot" ];
  then
    SESSION_TYPE="LoginWindow";
  fi
  
  ${COMMAND_SU} -l ${USERNAME} -c "${COMMAND_LAUNCHCTL} load -w -S ${SESSION_TYPE} ${SERVICE_PATH}"

  return 0
}

loadDaemonFile()
{
  if [ ! -f "${SERVICE_PATH}" ];
  then
    errorMsg "Local session plist file do not exist." "1"
  fi

  isDaemonRunning

  RUNNING=$?

  if [ "$RUNNING" -ne 0 ];
  then
    errorMsg "Local session is already loaded." "1"
  fi

  ${COMMAND_LAUNCHCTL} load -w ${SERVICE_PATH}

  return 0
}

nxhtdSetUsername()
{
  EXPRESSION="N;s|<key>Username</key>\s*\n\?<string>.*</string>|<key>Username</key><string>${USERNAME}</string>|";

  command="${COMMAND_SED} -e '${EXPRESSION}' '${SERVICE_PATH}' > '${SERVICE_PATH}.tmp'"

  ${COMMAND_ECHO} "NX> 900 Running command:[${command}]."
  result=`/bin/bash -c "${command}" 2>&1`

  if test $? != 0 -o "x$result" != "x";
  then
    errorMsg "Problems while updating: '${SERVICE_PATH}.tmp' file: [$!][${result}]."
  else
    ${COMMAND_ECHO} "NX> 900 File:'${SERVICE_PATH}.tmp' updated with no error result:[$!][${result}] exiting with:[$?]."
  fi

  if test -f "${SERVICE_PATH}.tmp";
  then
    ${COMMAND_CHMOD} 0644 "${SERVICE_PATH}.tmp"
    getRootGName
    ${COMMAND_CHOWN} root:${ROOTGNAME} "${SERVICE_PATH}.tmp"
    ${COMMAND_ECHO} "NX> 900 moving '${SERVICE_PATH}.tmp' in '${SERVICE_PATH}'."
    command="${COMMAND_MV} -f '${SERVICE_PATH}.tmp' '${SERVICE_PATH}'"
    result=`/bin/bash -c "${command}" 2>&1`
  fi

  if test $? != 0 -o "x${result}" != "x";
  then
    errorMsg "Problems while moving: '${SERVICE_PATH}.tmp' file: [$!][${result}]."
    #add here the recovery of backuppped file procedure.
    exit 1
  else
    ${COMMAND_ECHO} "NX> 900 File:'${SERVICE_PATH}.tmp' moved in '${SERVICE_PATH}' with no error result:[$!][${result}] exiting with:[$?]."
  fi                                   
}

case "$3" in
'--copy')               copyFile    ;;
'--load')               loadFile    ;;
'--loadDaemon')         loadDaemonFile    ;;
'--nxhtdSetUsername')   nxhtdSetUsername    ;;
esac

